@@ -112,7 +112,7 @@ int PrintControlVars( FILE *fp )
 {
     int          i, num_cvar, nameLen, verbosity, descLen, binding, scope;
     int          hasValue;
-    char         name[MAX_NAME_LEN], desc[MAX_DESC_LEN], varValue[21];
+    char         name[MAX_NAME_LEN], desc[MAX_DESC_LEN], varValue[512];
     MPI_T_enum   enumtype;
     MPI_Datatype datatype;
 
@@ -166,7 +166,7 @@ int PrintPerfVars( FILE *fp )
 {
     int          i, numPvar, nameLen, descLen, verbosity, varClass;
     int          binding, isReadonly, isContinuous, isAtomic;
-    char         name[MAX_NAME_LEN], desc[MAX_DESC_LEN], varValue[21];
+    char         name[MAX_NAME_LEN], desc[MAX_DESC_LEN], varValue[512];
     MPI_T_enum   enumtype;
     MPI_Datatype datatype;
 
@@ -468,33 +468,25 @@ int getCvarValueAsStr( int idx, MPI_Datatype datatype,
 
     MPI_T_cvar_handle_alloc( idx, NULL, &chandle, &count );
     if (count == 1 || (datatype==MPI_CHAR && count < varValueLen)) {
-        switch (datatype) {
-        case MPI_INT:
+        if (MPI_INT == datatype) {
             MPI_T_cvar_read( chandle, &ival );
             snprintf( varValue, varValueLen, "%d", ival );
             hasValue = 1;
-            break;
-        case MPI_UNSIGNED:
+        } else if (MPI_UNSIGNED == datatype) {
             MPI_T_cvar_read( chandle, &uval );
             snprintf( varValue, varValueLen, "%u", uval );
             hasValue = 1;
-            break;
-        case MPI_UNSIGNED_LONG:
+        } else if (MPI_UNSIGNED_LONG == datatype) {
             MPI_T_cvar_read( chandle, &ulval );
             snprintf( varValue, varValueLen, "%lu", ulval );
             hasValue = 1;
-            break;
-        case MPI_UNSIGNED_LONG_LONG:
+        } else if (MPI_UNSIGNED_LONG_LONG == datatype) {
             MPI_T_cvar_read( chandle, &ullval );
             snprintf( varValue, varValueLen, "%llu", ullval );
             hasValue = 1;
-            break;
-        case MPI_CHAR:
+        } else if (MPI_CHAR == datatype) {
             MPI_T_cvar_read( chandle, varValue );
             hasValue = 1;
-            break;
-        default:
-            break;
         }
     }
     MPI_T_cvar_handle_free( &chandle );
@@ -534,38 +526,29 @@ int getPvarValueAsStr( int idx, MPI_Datatype datatype, int isContinuous,
                 goto fn_fail;
             }
         }
-        switch (datatype) {
-        case MPI_INT:
+        if (MPI_INT == datatype) {
             MPI_T_pvar_read( session, phandle, &ival );
             snprintf( varValue, varValueLen, "%d", ival );
             hasValue = 1;
-            break;
-        case MPI_UNSIGNED:
+        } else if (MPI_UNSIGNED == datatype) {
             MPI_T_pvar_read( session, phandle, &uval );
             snprintf( varValue, varValueLen, "%u", uval );
             hasValue = 1;
-            break;
-        case MPI_UNSIGNED_LONG:
+        } else if (MPI_UNSIGNED_LONG == datatype) {
             MPI_T_pvar_read( session, phandle, &ulval );
             snprintf( varValue, varValueLen, "%lu", ulval );
             hasValue = 1;
-            break;
-        case MPI_UNSIGNED_LONG_LONG:
+        } else if (MPI_UNSIGNED_LONG_LONG == datatype) {
             MPI_T_pvar_read( session, phandle, &ullval );
             snprintf( varValue, varValueLen, "%llu", ullval );
             hasValue = 1;
-            break;
-        case MPI_DOUBLE:
+        } else if (MPI_DOUBLE == datatype) {
             MPI_T_pvar_read( session, phandle, &dval );
             snprintf( varValue, varValueLen, "%e", dval );
             hasValue = 1;
-            break;
-        case MPI_CHAR:
+        } else if (MPI_CHAR == datatype) {
             MPI_T_pvar_read( session, phandle, varValue );
             hasValue = 1;
-            break;
-        default:
-            break;
         }
     }
  fn_fail:
