@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -8,6 +8,32 @@
 #include "mpiimpl.h"
 #include "datatype.h"
 
+/*
+=== BEGIN_MPI_T_CVAR_INFO_BLOCK ===
+
+categories:
+    - name        : FAULT_TOLERANCE
+      description : cvars that control fault tolerance behavior
+
+cvars:
+    - name        : MPIR_CVAR_ENABLE_COLL_FT_RET
+      category    : FAULT_TOLERANCE
+      type        : boolean
+      default     : true
+      class       : device
+      verbosity   : MPI_T_VERBOSITY_USER_BASIC
+      scope       : MPI_T_SCOPE_ALL_EQ
+      description : >-
+        DEPRECATED! Will be removed in MPICH-3.2
+        Collectives called on a communicator with a failed process
+        should not hang, however the result of the operation may be
+        invalid even though the function returns MPI_SUCCESS.  This
+        option enables an experimental feature that will return an error
+        if the result of the collective is invalid.
+
+=== END_MPI_T_CVAR_INFO_BLOCK ===
+*/
+
 #define COPY_BUFFER_SZ 16384
 
 /* These functions are used in the implementation of collective
@@ -40,274 +66,6 @@ int MPIC_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIC_Send
-#undef FCNAME
-#define FCNAME "MPIC_Send"
-int MPIC_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
-              MPI_Comm comm)
-{
-    int mpi_errno, context_id;
-    MPID_Request *request_ptr=NULL;
-    MPID_Comm *comm_ptr=NULL;
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_SEND);
-
-    MPIDI_PT2PT_FUNC_ENTER_FRONT(MPID_STATE_MPIC_SEND);
-
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Send(buf, count, datatype, dest, tag, comm_ptr,
-                          context_id, &request_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-    if (request_ptr) {
-        mpi_errno = MPIC_Wait(request_ptr);
-	if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-	MPID_Request_release(request_ptr);
-    }
- fn_exit:
-    MPIDI_PT2PT_FUNC_EXIT(MPID_STATE_MPIC_SEND);
-    return mpi_errno;
- fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    if (request_ptr) {
-        MPID_Request_release(request_ptr);
-    }
-    goto fn_exit;
-    /* --END ERROR HANDLING-- */
-}
-
-#undef FUNCNAME
-#define FUNCNAME MPIC_Recv
-#undef FCNAME
-#define FCNAME "MPIC_Recv"
-int MPIC_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag,
-	     MPI_Comm comm, MPI_Status *status)
-{
-    int mpi_errno, context_id;
-    MPID_Request *request_ptr=NULL;
-    MPID_Comm *comm_ptr = NULL;
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_RECV);
-
-    MPIDI_PT2PT_FUNC_ENTER_BACK(MPID_STATE_MPIC_RECV);
-
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Recv(buf, count, datatype, source, tag, comm_ptr,
-                          context_id, status, &request_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-    if (request_ptr) {
-        mpi_errno = MPIC_Wait(request_ptr);
-	if (mpi_errno == MPI_SUCCESS) {
-	    if (status != MPI_STATUS_IGNORE) {
-		*status = request_ptr->status;
-	    }
-	    mpi_errno = request_ptr->status.MPI_ERROR;
-	}
-	else { MPIU_ERR_POP(mpi_errno); }
-
-        MPID_Request_release(request_ptr);
-    }
- fn_exit:
-    MPIDI_PT2PT_FUNC_EXIT_BACK(MPID_STATE_MPIC_RECV);
-    return mpi_errno;
- fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    if (request_ptr) { 
-	MPID_Request_release(request_ptr);
-    }
-    goto fn_exit;
-    /* --END ERROR HANDLING-- */
-}
-
-#undef FUNCNAME
-#define FUNCNAME MPIC_Ssend
-#undef FCNAME
-#define FCNAME "MPIC_Ssend"
-int MPIC_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
-               MPI_Comm comm)
-{
-    int mpi_errno, context_id;
-    MPID_Request *request_ptr=NULL;
-    MPID_Comm *comm_ptr=NULL;
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_SSEND);
-
-    MPIDI_PT2PT_FUNC_ENTER_FRONT(MPID_STATE_MPIC_SSEND);
-
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Ssend(buf, count, datatype, dest, tag, comm_ptr,
-                           context_id, &request_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-    if (request_ptr) {
-        mpi_errno = MPIC_Wait(request_ptr);
-	if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-	MPID_Request_release(request_ptr);
-    }
- fn_exit:
-    MPIDI_PT2PT_FUNC_EXIT(MPID_STATE_MPIC_SSEND);
-    return mpi_errno;
- fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    if (request_ptr) {
-        MPID_Request_release(request_ptr);
-    }
-    goto fn_exit;
-    /* --END ERROR HANDLING-- */
-}
-
-#undef FUNCNAME
-#define FUNCNAME MPIC_Sendrecv
-#undef FCNAME
-#define FCNAME "MPIC_Sendrecv"
-int MPIC_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
-                  int dest, int sendtag, void *recvbuf, int recvcount,
-                  MPI_Datatype recvtype, int source, int recvtag,
-                  MPI_Comm comm, MPI_Status *status) 
-{
-    MPID_Request *recv_req_ptr=NULL, *send_req_ptr=NULL;
-    int mpi_errno, context_id;
-    MPID_Comm *comm_ptr = NULL;
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_SENDRECV);
-
-    MPIDI_PT2PT_FUNC_ENTER_BOTH(MPID_STATE_MPIC_SENDRECV);
-
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Irecv(recvbuf, recvcount, recvtype, source, recvtag,
-                           comm_ptr, context_id, &recv_req_ptr);
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-    mpi_errno = MPID_Isend(sendbuf, sendcount, sendtype, dest, sendtag, 
-                           comm_ptr, context_id, &send_req_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-
-    mpi_errno = MPIC_Wait(send_req_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-    
-    mpi_errno = MPIC_Wait(recv_req_ptr);
-    if (mpi_errno) { MPIU_ERR_POPFATAL(mpi_errno); }
-    if (status != MPI_STATUS_IGNORE)
-        *status = recv_req_ptr->status;
-    mpi_errno = recv_req_ptr->status.MPI_ERROR;
-
-    MPID_Request_release(send_req_ptr);
-    MPID_Request_release(recv_req_ptr);
- fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    MPIDI_PT2PT_FUNC_EXIT_BOTH(MPID_STATE_MPIC_SENDRECV);
-    return mpi_errno;
-    /* --END ERROR HANDLING-- */
-}
-
-/* NOTE: for regular collectives (as opposed to irregular collectives) calling
- * this function repeatedly will almost always be slower than performing the
- * equivalent inline because of the overhead of the repeated malloc/free */
-#undef FUNCNAME
-#define FUNCNAME MPIC_Sendrecv_replace
-#undef FCNAME
-#define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype,
-                          int dest, int sendtag,
-                          int source, int recvtag,
-                          MPI_Comm comm, MPI_Status *status)
-{
-    int mpi_errno = MPI_SUCCESS;
-    MPIR_Context_id_t context_id_offset;
-    MPID_Request *sreq;
-    MPID_Request *rreq;
-    void *tmpbuf = NULL;
-    int tmpbuf_size = 0;
-    int tmpbuf_count = 0;
-    MPID_Comm *comm_ptr;
-    MPIU_CHKLMEM_DECL(1);
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_SENDRECV_REPLACE);
-#ifdef MPID_LOG_ARROWS
-    /* The logging macros log sendcount and recvcount */
-    int sendcount = count, recvcount = count;
-#endif
-
-    MPIDI_PT2PT_FUNC_ENTER_BOTH(MPID_STATE_MPIC_SENDRECV_REPLACE);
-    
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id_offset = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    if (count > 0 && dest != MPI_PROC_NULL)
-    {
-        MPIR_Pack_size_impl(count, datatype, &tmpbuf_size);
-        MPIU_CHKLMEM_MALLOC(tmpbuf, void *, tmpbuf_size, mpi_errno, "temporary send buffer");
-
-        mpi_errno = MPIR_Pack_impl(buf, count, datatype, tmpbuf, tmpbuf_size, &tmpbuf_count);
-        if (mpi_errno) MPIU_ERR_POP(mpi_errno);
-    }
-
-    mpi_errno = MPID_Irecv(buf, count, datatype, source, recvtag,
-                           comm_ptr, context_id_offset, &rreq);
-    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
-
-    mpi_errno = MPID_Isend(tmpbuf, tmpbuf_count, MPI_PACKED, dest,
-                           sendtag, comm_ptr, context_id_offset,
-                           &sreq);
-    if (mpi_errno != MPI_SUCCESS)
-    {
-        /* --BEGIN ERROR HANDLING-- */
-        /* FIXME: should we cancel the pending (possibly completed) receive request or wait for it to complete? */
-        MPID_Request_release(rreq);
-        MPIU_ERR_POP(mpi_errno);
-        /* --END ERROR HANDLING-- */
-    }
-
-    if (!MPID_Request_is_complete(sreq) || !MPID_Request_is_complete(rreq))
-    {
-        MPID_Progress_state progress_state;
-
-        MPID_Progress_start(&progress_state);
-        while (!MPID_Request_is_complete(sreq) || !MPID_Request_is_complete(rreq))
-        {
-            mpi_errno = MPID_Progress_wait(&progress_state);
-            if (mpi_errno != MPI_SUCCESS)
-            {
-                /* --BEGIN ERROR HANDLING-- */
-                MPID_Progress_end(&progress_state);
-                MPIU_ERR_POP(mpi_errno);
-                /* --END ERROR HANDLING-- */
-            }
-        }
-        MPID_Progress_end(&progress_state);
-    }
-
-    if (status != MPI_STATUS_IGNORE) {
-        *status = rreq->status;
-    }
-
-    if (mpi_errno == MPI_SUCCESS) {
-        mpi_errno = rreq->status.MPI_ERROR;
-
-        if (mpi_errno == MPI_SUCCESS) {
-            mpi_errno = sreq->status.MPI_ERROR;
-        }
-    }
-
-    MPID_Request_release(sreq);
-    MPID_Request_release(rreq);
-
-    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
-
-fn_exit:
-    MPIU_CHKLMEM_FREEALL();
-    MPIDI_PT2PT_FUNC_EXIT_BOTH(MPID_STATE_MPIC_SENDRECV_REPLACE);
-    return mpi_errno;
-fn_fail:
-    goto fn_exit;
-}
-
 #undef FUNCNAME
 #define FUNCNAME MPIR_Localcopy
 #undef FCNAME
@@ -324,27 +82,35 @@ int MPIR_Localcopy(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
 
     MPID_MPI_FUNC_ENTER(MPID_STATE_MPIR_LOCALCOPY);
 
-    MPIR_Datatype_iscontig(sendtype, &sendtype_iscontig);
-    MPIR_Datatype_iscontig(recvtype, &recvtype_iscontig);
-
     MPID_Datatype_get_size_macro(sendtype, sendsize);
     MPID_Datatype_get_size_macro(recvtype, recvsize);
+
     sdata_sz = sendsize * sendcount;
     rdata_sz = recvsize * recvcount;
 
+    /* if there is no data to copy, bail out */
     if (!sdata_sz || !rdata_sz)
         goto fn_exit;
-    
-    if (sdata_sz > rdata_sz)
-    {
+
+#if defined(HAVE_ERROR_CHECKING)
+    if (sdata_sz > rdata_sz) {
         MPIU_ERR_SET2(mpi_errno, MPI_ERR_TRUNCATE, "**truncate", "**truncate %d %d", sdata_sz, rdata_sz);
         copy_sz = rdata_sz;
     }
     else
-    {
+#endif /* HAVE_ERROR_CHECKING */
         copy_sz = sdata_sz;
+
+    /* Builtin types is the common case; optimize for it */
+    if ((HANDLE_GET_KIND(sendtype) == HANDLE_KIND_BUILTIN) &&
+        HANDLE_GET_KIND(recvtype) == HANDLE_KIND_BUILTIN) {
+        MPIU_Memcpy(recvbuf, sendbuf, copy_sz);
+        goto fn_exit;
     }
 
+    MPIR_Datatype_iscontig(sendtype, &sendtype_iscontig);
+    MPIR_Datatype_iscontig(recvtype, &recvtype_iscontig);
+
     MPIR_Type_get_true_extent_impl(sendtype, &sendtype_true_lb, &true_extent);
     MPIR_Type_get_true_extent_impl(recvtype, &recvtype_true_lb, &true_extent);
 
@@ -452,69 +218,6 @@ int MPIR_Localcopy(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIC_Isend
-#undef FCNAME
-#define FCNAME "MPIC_Isend"
-int MPIC_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
-              MPI_Comm comm, MPI_Request *request)
-{
-    int mpi_errno, context_id;
-    MPID_Request *request_ptr=NULL;
-    MPID_Comm *comm_ptr=NULL;
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_ISEND);
-
-    MPIDI_PT2PT_FUNC_ENTER_FRONT(MPID_STATE_MPIC_ISEND);
-
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Isend(buf, count, datatype, dest, tag, comm_ptr,
-                          context_id, &request_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); } 
-
-    *request = request_ptr->handle;
-
- fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    MPIDI_PT2PT_FUNC_EXIT(MPID_STATE_MPIC_ISEND);
-    return mpi_errno;
-    /* --END ERROR HANDLING-- */
-}
-
-
-#undef FUNCNAME
-#define FUNCNAME MPIC_Irecv
-#undef FCNAME
-#define FCNAME "MPIC_Irecv"
-int MPIC_Irecv(void *buf, int count, MPI_Datatype datatype, int
-               source, int tag, MPI_Comm comm, MPI_Request *request)
-{
-    int mpi_errno, context_id;
-    MPID_Request *request_ptr=NULL;
-    MPID_Comm *comm_ptr = NULL;
-    MPIDI_STATE_DECL(MPID_STATE_MPIC_IRECV);
-
-    MPIDI_PT2PT_FUNC_ENTER_BACK(MPID_STATE_MPIC_IRECV);
-
-    MPID_Comm_get_ptr( comm, comm_ptr );
-    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
-        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
-
-    mpi_errno = MPID_Irecv(buf, count, datatype, source, tag, comm_ptr,
-                          context_id, &request_ptr); 
-    if (mpi_errno) { MPIU_ERR_POP(mpi_errno); }
-
-    *request = request_ptr->handle;
-
- fn_fail:
-    /* --BEGIN ERROR HANDLING-- */
-    MPIDI_PT2PT_FUNC_EXIT_BACK(MPID_STATE_MPIC_IRECV);
-    return mpi_errno;
-    /* --END ERROR HANDLING-- */
-}
-
 /* FIXME: For the brief-global and finer-grain control, we must ensure that
    the global lock is *not* held when this routine is called. (unless we change
    progress_start/end to grab the lock, in which case we must *still* make
@@ -571,64 +274,100 @@ int MPIC_Wait(MPID_Request * request_ptr)
    this is OK since there is no data that can be received corrupted. */
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Send_ft
+#define FUNCNAME MPIC_Send
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Send_ft(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
+int MPIC_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
                  MPI_Comm comm, int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
+    int context_id;
+    MPID_Request *request_ptr = NULL;
+    MPID_Comm *comm_ptr = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIC_SEND_FT);
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_SEND_FT);
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
 
-    if (*errflag && MPIR_PARAM_ENABLE_COLL_FT_RET)
-        mpi_errno = MPIC_Send(buf, count, datatype, dest, MPIR_ERROR_TAG, comm);
-    else
-        mpi_errno = MPIC_Send(buf, count, datatype, dest, tag, comm);
+    MPIU_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", count);
+
+    if (*errflag && MPIR_CVAR_ENABLE_COLL_FT_RET)
+        MPIR_TAG_SET_ERROR_BIT(tag);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
+    mpi_errno = MPID_Send(buf, count, datatype, dest, tag, comm_ptr,
+                          context_id, &request_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+    if (request_ptr) {
+        mpi_errno = MPIC_Wait(request_ptr);
+        if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+        MPID_Request_release(request_ptr);
+    }
 
  fn_exit:
     MPIDI_FUNC_EXIT(MPID_STATE_MPIC_SEND_FT);
     return mpi_errno;
  fn_fail:
+    /* --BEGIN ERROR HANDLING-- */
+    if (request_ptr) MPID_Request_release(request_ptr);
     goto fn_exit;
+    /* --END ERROR HANDLING-- */
 }
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Recv_ft
+#define FUNCNAME MPIC_Recv
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Recv_ft(void *buf, int count, MPI_Datatype datatype, int source, int tag,
+int MPIC_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag,
                  MPI_Comm comm, MPI_Status *status, int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
+    int context_id;
     MPI_Status mystatus;
+    MPID_Request *request_ptr = NULL;
+    MPID_Comm *comm_ptr = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIC_RECV_FT);
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_RECV_FT);
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
 
-    if (!MPIR_PARAM_ENABLE_COLL_FT_RET) {
-            mpi_errno = MPIC_Recv(buf, count, datatype, source, tag, comm, status);
-            goto fn_exit;
-    }
-    
+    MPIU_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", count);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
     if (status == MPI_STATUS_IGNORE)
         status = &mystatus;
-    
-    mpi_errno = MPIC_Recv(buf, count, datatype, source, MPI_ANY_TAG, comm, status);
-    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
 
-    if (*errflag)
-        goto fn_exit;
+    mpi_errno = MPID_Recv(buf, count, datatype, source, tag, comm_ptr,
+                          context_id, status, &request_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+    if (request_ptr) {
+        mpi_errno = MPIC_Wait(request_ptr);
+        if (mpi_errno == MPI_SUCCESS) {
+            *status = request_ptr->status;
+            mpi_errno = request_ptr->status.MPI_ERROR;
+        } else {
+            MPIU_ERR_POP(mpi_errno);
+        }
+        MPID_Request_release(request_ptr);
+    }
+
+    if (!MPIR_CVAR_ENABLE_COLL_FT_RET) goto fn_exit;
 
     if (source != MPI_PROC_NULL) {
-        if (status->MPI_TAG == MPIR_ERROR_TAG)
+        if (MPIR_TAG_CHECK_ERROR_BIT(status->MPI_TAG)) {
             *errflag = TRUE;
-        else {
+            MPIR_TAG_CLEAR_ERROR_BIT(status->MPI_TAG);
+        } else if (MPIX_ERR_REVOKED != MPIR_ERR_GET_CLASS(status->MPI_ERROR)) {
             MPIU_Assert(status->MPI_TAG == tag);
         }
     }
@@ -638,82 +377,120 @@ int MPIC_Recv_ft(void *buf, int count, MPI_Datatype datatype, int source, int ta
     MPIDI_FUNC_EXIT(MPID_STATE_MPIC_RECV_FT);
     return mpi_errno;
  fn_fail:
+    /* --BEGIN ERROR HANDLING-- */
+    if (request_ptr) MPID_Request_release(request_ptr);
     goto fn_exit;
+    /* --END ERROR HANDLING-- */
 }
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Ssend_ft
+#define FUNCNAME MPIC_Ssend
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Ssend_ft(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
+int MPIC_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
                   MPI_Comm comm, int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
+    int context_id;
+    MPID_Request *request_ptr = NULL;
+    MPID_Comm *comm_ptr = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIC_SSEND_FT);
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_SSEND_FT);
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
-    
-    if (*errflag && MPIR_PARAM_ENABLE_COLL_FT_RET)
-        mpi_errno = MPIC_Ssend(buf, count, datatype, dest, MPIR_ERROR_TAG, comm);
-    else
-        mpi_errno = MPIC_Ssend(buf, count, datatype, dest, tag, comm);
+
+    MPIU_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
+            "**countneg", "**countneg %d", count);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
+    if (*errflag && MPIR_CVAR_ENABLE_COLL_FT_RET)
+        MPIR_TAG_SET_ERROR_BIT(tag);
+
+    mpi_errno = MPID_Ssend(buf, count, datatype, dest, tag, comm_ptr,
+                           context_id, &request_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+    if (request_ptr) {
+        mpi_errno = MPIC_Wait(request_ptr);
+        if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+        MPID_Request_release(request_ptr);
+    }
 
  fn_exit:
     MPIDI_FUNC_EXIT(MPID_STATE_MPIC_SSEND_FT);
     return mpi_errno;
  fn_fail:
+    /* --BEGIN ERROR HANDLING-- */
+    if (request_ptr) MPID_Request_release(request_ptr);
     goto fn_exit;
+    /* --END ERROR HANDLING-- */
 }
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Sendrecv_ft
+#define FUNCNAME MPIC_Sendrecv
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Sendrecv_ft(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
+int MPIC_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
                      int dest, int sendtag, void *recvbuf, int recvcount,
                      MPI_Datatype recvtype, int source, int recvtag,
                      MPI_Comm comm, MPI_Status *status, int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
+    int context_id;
     MPI_Status mystatus;
+    MPID_Request *recv_req_ptr = NULL, *send_req_ptr = NULL;
+    MPID_Comm *comm_ptr = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIC_SENDRECV_FT);
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_SENDRECV_FT);
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
 
-    if (!MPIR_PARAM_ENABLE_COLL_FT_RET) {
-        mpi_errno = MPIC_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag,
-                                  recvbuf, recvcount, recvtype, source, recvtag,
-                                  comm, status);
-        goto fn_exit;
-    }
-    
-    if (status == MPI_STATUS_IGNORE)
-        status = &mystatus;
-    
-    if (*errflag) {
-        mpi_errno = MPIC_Sendrecv(sendbuf, sendcount, sendtype, dest, MPIR_ERROR_TAG,
-                                  recvbuf, recvcount, recvtype, source, MPI_ANY_TAG,
-                                  comm, status);
-        goto fn_exit;
-    }
-    
-    mpi_errno = MPIC_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag,
-                              recvbuf, recvcount, recvtype, source, MPI_ANY_TAG,
-                              comm, status);
+    MPIU_ERR_CHKANDJUMP1((sendcount < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", sendcount);
+    MPIU_ERR_CHKANDJUMP1((recvcount < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", recvcount);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
+    if (status == MPI_STATUS_IGNORE) status = &mystatus;
+    if (MPIR_CVAR_ENABLE_COLL_FT_RET)
+        if (*errflag) MPIR_TAG_SET_ERROR_BIT(sendtag);
+
+    mpi_errno = MPID_Irecv(recvbuf, recvcount, recvtype, source, recvtag,
+                           comm_ptr, context_id, &recv_req_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+    mpi_errno = MPID_Isend(sendbuf, sendcount, sendtype, dest, recvtag,
+                           comm_ptr, context_id, &send_req_ptr);
     if (mpi_errno) MPIU_ERR_POP(mpi_errno);
 
+    mpi_errno = MPIC_Wait(send_req_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+    mpi_errno = MPIC_Wait(recv_req_ptr);
+    if (mpi_errno) MPIU_ERR_POPFATAL(mpi_errno);
+
+    *status = recv_req_ptr->status;
+    mpi_errno = recv_req_ptr->status.MPI_ERROR;
+
+    MPID_Request_release(send_req_ptr);
+    MPID_Request_release(recv_req_ptr);
+
+    if (!MPIR_CVAR_ENABLE_COLL_FT_RET) goto fn_exit;
+
     if (source != MPI_PROC_NULL) {
-        if (status->MPI_TAG == MPIR_ERROR_TAG)
+        if (MPIR_TAG_CHECK_ERROR_BIT(status->MPI_TAG)) {
             *errflag = TRUE;
-        else {
+            MPIR_TAG_CLEAR_ERROR_BIT(status->MPI_TAG);
+        } else if (MPIX_ERR_REVOKED != MPIR_ERR_GET_CLASS(status->MPI_ERROR)) {
             MPIU_Assert(status->MPI_TAG == recvtag);
         }
     }
-    
+
  fn_exit:
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "OUT: errflag = %s", *errflag?"TRUE":"FALSE");
 
@@ -723,57 +500,115 @@ int MPIC_Sendrecv_ft(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
     goto fn_exit;
 }
 
+/* NOTE: for regular collectives (as opposed to irregular collectives) calling
+ * this function repeatedly will almost always be slower than performing the
+ * equivalent inline because of the overhead of the repeated malloc/free */
 #undef FUNCNAME
-#define FUNCNAME MPIC_Sendrecv_replace_ft
+#define FUNCNAME MPIC_Sendrecv_replace
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Sendrecv_replace_ft(void *buf, int count, MPI_Datatype datatype,
+int MPIC_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype,
                              int dest, int sendtag,
                              int source, int recvtag,
                              MPI_Comm comm, MPI_Status *status, int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
     MPI_Status mystatus;
+    MPIR_Context_id_t context_id_offset;
+    MPID_Request *sreq;
+    MPID_Request *rreq;
+    void *tmpbuf = NULL;
+    MPI_Aint tmpbuf_size = 0;
+    MPI_Aint tmpbuf_count = 0;
+    MPID_Comm *comm_ptr;
+    MPIU_CHKLMEM_DECL(1);
     MPIDI_STATE_DECL(MPID_STATE_MPIC_SENDRECV_REPLACE_FT);
+#ifdef MPID_LOG_ARROWS
+    /* The logging macros log sendcount and recvcount */
+    int sendcount = count, recvcount = count;
+#endif
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_SENDRECV_REPLACE_FT);
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
 
-    if (!MPIR_PARAM_ENABLE_COLL_FT_RET) {
-        mpi_errno = MPIC_Sendrecv_replace(buf, count, datatype,
-                                          dest, sendtag,
-                                          source, recvtag,
-                                          comm, status);
-        goto fn_exit;
+    MPIU_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", count);
+
+    if (status == MPI_STATUS_IGNORE) status = &mystatus;
+    if (MPIR_CVAR_ENABLE_COLL_FT_RET)
+        if (*errflag) MPIR_TAG_SET_ERROR_BIT(sendtag);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id_offset = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
+    if (count > 0 && dest != MPI_PROC_NULL) {
+        MPIR_Pack_size_impl(count, datatype, &tmpbuf_size);
+        MPIU_CHKLMEM_MALLOC(tmpbuf, void *, tmpbuf_size, mpi_errno, "temporary send buffer");
+
+        mpi_errno = MPIR_Pack_impl(buf, count, datatype, tmpbuf, tmpbuf_size, &tmpbuf_count);
+        if (mpi_errno) MPIU_ERR_POP(mpi_errno);
     }
 
-    if (status == MPI_STATUS_IGNORE)
-        status = &mystatus;
-    
-    if (*errflag) {
-        mpi_errno = MPIC_Sendrecv_replace(buf, count, datatype,
-                                          dest, MPIR_ERROR_TAG,
-                                          source, MPI_ANY_TAG,
-                                          comm, status);
-        goto fn_exit;
+    mpi_errno = MPID_Irecv(buf, count, datatype, source, recvtag,
+                           comm_ptr, context_id_offset, &rreq);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+
+    mpi_errno = MPID_Isend(tmpbuf, tmpbuf_count, MPI_PACKED, dest,
+                           sendtag, comm_ptr, context_id_offset, &sreq);
+    if (mpi_errno != MPI_SUCCESS) {
+        /* --BEGIN ERROR HANDLING-- */
+        /* FIXME: should we cancel the pending (possibly completed) receive
+         * request or wait for it to complete? */
+        MPID_Request_release(rreq);
+        MPIU_ERR_POP(mpi_errno);
+        /* --END ERROR HANDLING-- */
     }
-    
-    mpi_errno = MPIC_Sendrecv_replace(buf, count, datatype,
-                                      dest, sendtag,
-                                      source, MPI_ANY_TAG,
-                                      comm, status);
+
+    if (!MPID_Request_is_complete(sreq) || !MPID_Request_is_complete(rreq)) {
+        MPID_Progress_state progress_state;
+
+        MPID_Progress_start(&progress_state);
+        while (!MPID_Request_is_complete(sreq) || !MPID_Request_is_complete(rreq)) {
+            mpi_errno = MPID_Progress_wait(&progress_state);
+            if (mpi_errno != MPI_SUCCESS) {
+                /* --BEGIN ERROR HANDLING-- */
+                MPID_Progress_end(&progress_state);
+                MPIU_ERR_POP(mpi_errno);
+                /* --END ERROR HANDLING-- */
+            }
+        }
+        MPID_Progress_end(&progress_state);
+    }
+
+    *status = rreq->status;
+
+    if (mpi_errno == MPI_SUCCESS) {
+        mpi_errno = rreq->status.MPI_ERROR;
+
+        if (mpi_errno == MPI_SUCCESS) {
+            mpi_errno = sreq->status.MPI_ERROR;
+        }
+    }
+
+    MPID_Request_release(sreq);
+    MPID_Request_release(rreq);
+
+    if (!MPIR_CVAR_ENABLE_COLL_FT_RET) goto fn_exit;
     if (mpi_errno) MPIU_ERR_POP(mpi_errno);
     
     if (source != MPI_PROC_NULL) {
-        if (status->MPI_TAG == MPIR_ERROR_TAG)
+        if (MPIR_TAG_CHECK_ERROR_BIT(status->MPI_TAG)) {
             *errflag = TRUE;
-        else {
+            MPIR_TAG_CLEAR_ERROR_BIT(status->MPI_TAG);
+        } else if (MPIX_ERR_REVOKED != MPIR_ERR_GET_CLASS(status->MPI_ERROR)) {
             MPIU_Assert(status->MPI_TAG == recvtag);
         }
     }
 
  fn_exit:
+    MPIU_CHKLMEM_FREEALL();
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "OUT: errflag = %s", *errflag?"TRUE":"FALSE");
     MPIDI_FUNC_EXIT(MPID_STATE_MPIC_SENDRECV_REPLACE_FT);
     return mpi_errno;
@@ -782,23 +617,37 @@ int MPIC_Sendrecv_replace_ft(void *buf, int count, MPI_Datatype datatype,
 }
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Isend_ft
+#define FUNCNAME MPIC_Isend
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Isend_ft(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
+int MPIC_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag,
                   MPI_Comm comm, MPI_Request *request, int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
+    int context_id;
+    MPID_Request *request_ptr = NULL;
+    MPID_Comm *comm_ptr = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIC_ISEND_FT);
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_ISEND_FT);
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
 
-    if (*errflag && MPIR_PARAM_ENABLE_COLL_FT_RET)
-        mpi_errno = MPIC_Isend(buf, count, datatype, dest, MPIR_ERROR_TAG, comm, request);
-    else
-        mpi_errno = MPIC_Isend(buf, count, datatype, dest, tag, comm, request);
+    MPIU_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", count);
+
+    if (*errflag && MPIR_CVAR_ENABLE_COLL_FT_RET)
+        MPIR_TAG_SET_ERROR_BIT(tag);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
+    mpi_errno = MPID_Isend(buf, count, datatype, dest, tag, comm_ptr,
+            context_id, &request_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+
+    *request = request_ptr->handle;
 
  fn_exit:
     MPIDI_FUNC_EXIT(MPID_STATE_MPIC_ISEND_FT);
@@ -808,21 +657,32 @@ int MPIC_Isend_ft(const void *buf, int count, MPI_Datatype datatype, int dest, i
 }
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Irecv_ft
+#define FUNCNAME MPIC_Irecv
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Irecv_ft(void *buf, int count, MPI_Datatype datatype, int source,
+int MPIC_Irecv(void *buf, int count, MPI_Datatype datatype, int source,
                   int tag, MPI_Comm comm, MPI_Request *request)
 {
     int mpi_errno = MPI_SUCCESS;
+    int context_id;
+    MPID_Request *request_ptr = NULL;
+    MPID_Comm *comm_ptr = NULL;
     MPIDI_STATE_DECL(MPID_STATE_MPIC_IRECV_FT);
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIC_IRECV_FT);
 
-    if (MPIR_PARAM_ENABLE_COLL_FT_RET)
-        mpi_errno = MPIC_Irecv(buf, count, datatype, source, MPI_ANY_TAG, comm, request);
-    else
-        mpi_errno = MPIC_Irecv(buf, count, datatype, source, tag, comm, request);
+    MPIU_ERR_CHKANDJUMP1((count < 0), mpi_errno, MPI_ERR_COUNT,
+                         "**countneg", "**countneg %d", count);
+
+    MPID_Comm_get_ptr(comm, comm_ptr);
+    context_id = (comm_ptr->comm_kind == MPID_INTRACOMM) ?
+        MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+
+    mpi_errno = MPID_Irecv(buf, count, datatype, source, tag, comm_ptr,
+            context_id, &request_ptr);
+    if (mpi_errno) MPIU_ERR_POP(mpi_errno);
+
+    *request = request_ptr->handle;
 
  fn_exit:
     MPIDI_FUNC_EXIT(MPID_STATE_MPIC_IRECV_FT);
@@ -833,10 +693,10 @@ int MPIC_Irecv_ft(void *buf, int count, MPI_Datatype datatype, int source,
 
 
 #undef FUNCNAME
-#define FUNCNAME MPIC_Waitall_ft
+#define FUNCNAME MPIC_Waitall
 #undef FCNAME
 #define FCNAME MPIU_QUOTE(FUNCNAME)
-int MPIC_Waitall_ft(int numreq, MPI_Request requests[], MPI_Status statuses[], int *errflag)
+int MPIC_Waitall(int numreq, MPI_Request requests[], MPI_Status statuses[], int *errflag)
 {
     int mpi_errno = MPI_SUCCESS;
     int i;
@@ -848,21 +708,22 @@ int MPIC_Waitall_ft(int numreq, MPI_Request requests[], MPI_Status statuses[], i
 
     MPIU_DBG_MSG_S(PT2PT, TYPICAL, "IN: errflag = %s", *errflag?"TRUE":"FALSE");
 
-    /* The MPI_TAG field is not set for send oeprations, so if we want
-       to check for MPIR_ERROR_TAG below, we should initialize all tag
-       fields here. */
+    /* The MPI_TAG field is not set for send operations, so if we want
+       to check for the error bit in the tag below, we should initialize all
+       tag fields here. */
     for (i = 0; i < numreq; ++i)
         statuses[i].MPI_TAG = 0;
     
     mpi_errno = MPIR_Waitall_impl(numreq, requests, statuses);
     if (mpi_errno) MPIU_ERR_POP(mpi_errno);
 
-    if (*errflag || !MPIR_PARAM_ENABLE_COLL_FT_RET)
+    if (*errflag || !MPIR_CVAR_ENABLE_COLL_FT_RET)
         goto fn_exit;
 
     for (i = 0; i < numreq; ++i) {
-        if (statuses[i].MPI_TAG == MPIR_ERROR_TAG) {
+        if (MPIR_TAG_CHECK_ERROR_BIT(statuses[i].MPI_TAG)) {
             *errflag = TRUE;
+            MPIR_TAG_CLEAR_ERROR_BIT(statuses[i].MPI_TAG);
             break;
         }
     }
