@@ -15,6 +15,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_Add_error_string  MPI_Add_error_string
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Add_error_string as PMPI_Add_error_string
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Add_error_string(int errorcode, const char *string) __attribute__((weak,alias("PMPI_Add_error_string")));
 #endif
 /* -- End Profiling Symbol Block */
 
