@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -14,6 +14,9 @@
 #pragma _HP_SECONDARY_DEF PMPI_Recv_init  MPI_Recv_init
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Recv_init as PMPI_Recv_init
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag,
+                  MPI_Comm comm, MPI_Request *request) __attribute__((weak,alias("PMPI_Recv_init")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -39,7 +42,7 @@ Input Parameters:
 . tag - message tag or 'MPI_ANY_TAG' (integer) 
 - comm - communicator (handle) 
 
-Output Parameter:
+Output Parameters:
 . request - communication request (handle) 
 
 .N ThreadSafe
@@ -90,7 +93,7 @@ int MPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source,
     {
         MPID_BEGIN_ERROR_CHECKS;
         {
-            MPID_Comm_valid_ptr( comm_ptr, mpi_errno );
+            MPID_Comm_valid_ptr( comm_ptr, mpi_errno, FALSE );
             if (mpi_errno) goto fn_fail;
 	    
 	    MPIR_ERRTEST_COUNT(count, mpi_errno);
