@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *
  *  (C) 2001 by Argonne National Laboratory.
@@ -14,6 +14,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_Test_cancelled  MPI_Test_cancelled
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Test_cancelled as PMPI_Test_cancelled
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Test_cancelled(const MPI_Status *status, int *flag) __attribute__((weak,alias("PMPI_Test_cancelled")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -32,10 +34,10 @@
 /*@
   MPI_Test_cancelled - Tests to see if a request was cancelled
 
-Input Parameter:
+Input Parameters:
 . status - status object (Status) 
 
-Output Parameter:
+Output Parameters:
 . flag - true if the request was cancelled, false otherwise (logical) 
 
 .N ThreadSafe
@@ -46,7 +48,7 @@ Output Parameter:
 .N MPI_SUCCESS
 .N MPI_ERR_ARG
 @*/
-int MPI_Test_cancelled(MPICH2_CONST MPI_Status *status, int *flag)
+int MPI_Test_cancelled(const MPI_Status *status, int *flag)
 {
     int mpi_errno = MPI_SUCCESS;
     MPID_MPI_STATE_DECL(MPID_STATE_MPI_TEST_CANCELLED);
