@@ -18,6 +18,9 @@
 #pragma _HP_SECONDARY_DEF PMPI_Waitsome  MPI_Waitsome
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Waitsome as PMPI_Waitsome
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount,
+                 int array_of_indices[], MPI_Status array_of_statuses[]) __attribute__((weak,alias("PMPI_Waitsome")));
 #endif
 /* -- End Profiling Symbol Block */
 
