@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /* 
  *
  *   Copyright (C) 1997 University of Chicago. 
@@ -16,6 +16,8 @@
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_File_set_atomicity as PMPI_File_set_atomicity
 /* end of weak pragmas */
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_File_set_atomicity(MPI_File fh, int flag) __attribute__((weak,alias("PMPI_File_set_atomicity")));
 #endif
 
 /* Include mapping from MPI->PMPI */
@@ -32,42 +34,40 @@ Input Parameters:
 
 .N fortran
 @*/
-int MPI_File_set_atomicity(MPI_File mpi_fh, int flag)
+int MPI_File_set_atomicity(MPI_File fh, int flag)
 {
     int error_code, tmp_flag;
     static char myname[] = "MPI_FILE_SET_ATOMICITY";
     ADIO_Fcntl_t *fcntl_struct;
-    ADIO_File fh;
-    MPIU_THREADPRIV_DECL;
+    ADIO_File adio_fh;
 
     MPIU_THREAD_CS_ENTER(ALLFUNC,);
-    MPIR_Nest_incr();
 
-    fh = MPIO_File_resolve(mpi_fh);
+    adio_fh = MPIO_File_resolve(fh);
 
     /* --BEGIN ERROR HANDLING-- */
-    MPIO_CHECK_FILE_HANDLE(fh, myname, error_code);
+    MPIO_CHECK_FILE_HANDLE(adio_fh, myname, error_code);
     /* --END ERROR HANDLING-- */
 
-    ADIOI_TEST_DEFERRED(fh, myname, &error_code);
+    ADIOI_TEST_DEFERRED(adio_fh, myname, &error_code);
 
     if (flag) flag = 1;  /* take care of non-one values! */
 
 /* check if flag is the same on all processes */
     tmp_flag = flag;
-    MPI_Bcast(&tmp_flag, 1, MPI_INT, 0, fh->comm);
+    MPI_Bcast(&tmp_flag, 1, MPI_INT, 0, adio_fh->comm);
 
     /* --BEGIN ERROR HANDLING-- */
     if (tmp_flag != flag) {
 	error_code = MPIO_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE,
 					  myname, __LINE__, MPI_ERR_ARG, 
 					  "**notsame", 0);
-	error_code = MPIO_Err_return_file(fh, error_code);
+	error_code = MPIO_Err_return_file(adio_fh, error_code);
 	goto fn_exit;
     }
     /* --END ERROR HANDLING-- */
 
-    if (fh->atomicity == flag){
+    if (adio_fh->atomicity == flag){
 	    error_code = MPI_SUCCESS;
 	    goto fn_exit;
     }
@@ -75,18 +75,17 @@ int MPI_File_set_atomicity(MPI_File mpi_fh, int flag)
 
     fcntl_struct = (ADIO_Fcntl_t *) ADIOI_Malloc(sizeof(ADIO_Fcntl_t));
     fcntl_struct->atomicity = flag;
-    ADIO_Fcntl(fh, ADIO_FCNTL_SET_ATOMICITY, fcntl_struct, &error_code);
+    ADIO_Fcntl(adio_fh, ADIO_FCNTL_SET_ATOMICITY, fcntl_struct, &error_code);
     /* TODO: what do we do with this error code? */
 
     /* --BEGIN ERROR HANDLING-- */
     if (error_code != MPI_SUCCESS)
-	error_code = MPIO_Err_return_file(fh, error_code);
+	error_code = MPIO_Err_return_file(adio_fh, error_code);
     /* --END ERROR HANDLING-- */
 
     ADIOI_Free(fcntl_struct);
 
 fn_exit:
-    MPIR_Nest_decr();
     MPIU_THREAD_CS_EXIT(ALLFUNC,);
     return error_code;
 }
