@@ -17,6 +17,9 @@
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_File_write_ordered as PMPI_File_write_ordered
 /* end of weak pragmas */
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype,
+                           MPI_Status *status) __attribute__((weak,alias("PMPI_File_write_ordered")));
 #endif
 
 /* Include mapping from MPI->PMPI */
