@@ -15,6 +15,10 @@
 #pragma _HP_SECONDARY_DEF PMPI_Dist_graph_create  MPI_Dist_graph_create
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_Dist_graph_create as PMPI_Dist_graph_create
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_Dist_graph_create(MPI_Comm comm_old, int n, const int sources[], const int degrees[],
+                          const int destinations[], const int weights[], MPI_Info info,
+                          int reorder, MPI_Comm *comm_dist_graph) __attribute__((weak,alias("PMPI_Dist_graph_create")));
 #endif
 /* -- End Profiling Symbol Block */
 
@@ -120,7 +124,7 @@ int MPI_Dist_graph_create(MPI_Comm comm_old, int n, const int sources[],
         MPID_BEGIN_ERROR_CHECKS;
         {
             /* Validate comm_ptr */
-            MPID_Comm_valid_ptr(comm_ptr, mpi_errno);
+            MPID_Comm_valid_ptr( comm_ptr, mpi_errno, FALSE );
             /* If comm_ptr is not valid, it will be reset to null */
             if (comm_ptr) {
                 MPIR_ERRTEST_COMM_INTRA(comm_ptr, mpi_errno);
