@@ -13,6 +13,8 @@
 #pragma _HP_SECONDARY_DEF PMPI_T_enum_get_info  MPI_T_enum_get_info
 #elif defined(HAVE_PRAGMA_CRI_DUP)
 #pragma _CRI duplicate MPI_T_enum_get_info as PMPI_T_enum_get_info
+#elif defined(HAVE_WEAK_ATTRIBUTE)
+int MPI_T_enum_get_info(MPI_T_enum enumtype, int *num, char *name, int *name_len) __attribute__((weak,alias("PMPI_T_enum_get_info")));
 #endif
 /* -- End Profiling Symbol Block */
 
