@@ -560,7 +560,7 @@ MPID_nem_mx_handle_rreq(MPID_Request *req, mx_status_t status)
   
   NEM_MX_MATCH_GET_RANK(match_info,req->status.MPI_SOURCE);
   NEM_MX_MATCH_GET_TAG(match_info,req->status.MPI_TAG);	   
-  req->status.count = status.xfer_length;	
+  MPIR_STATUS_SET_COUNT(req->status, status.xfer_length);
   req->dev.recv_data_sz = status.xfer_length;
   
   MPIDI_Datatype_get_info(req->dev.user_count, req->dev.datatype, dt_contig, userbuf_sz, dt_ptr, dt_true_lb);
@@ -596,7 +596,7 @@ MPID_nem_mx_handle_rreq(MPID_Request *req, mx_status_t status)
 						 "**truncate", "**truncate %d %d %d %d", 
 						 req->status.MPI_SOURCE, req->status.MPI_TAG, 
 						 req->dev.recv_data_sz, userbuf_sz );
-    req->status.count = userbuf_sz;
+    MPIR_STATUS_SET_COUNT(req->status, userbuf_sz);
     data_sz = userbuf_sz;
   }
   
@@ -607,7 +607,7 @@ MPID_nem_mx_handle_rreq(MPID_Request *req, mx_status_t status)
     MPID_Segment_unpack( req->dev.segment_ptr, 0, &last, req->dev.tmpbuf );
     MPIU_Free(req->dev.tmpbuf);       
     if (last != data_sz) {
-      req->status.count = (int)last;
+      MPIR_STATUS_SET_COUNT(req->status, last);
       if (req->dev.recv_data_sz <= userbuf_sz) {
 	MPIU_ERR_SETSIMPLE(req->status.MPI_ERROR,MPI_ERR_TYPE,"**dtypemismatch");
       }
