@@ -31,9 +31,12 @@ enum MPIDI_RMA_Datatype {
     MPIDI_RMA_DATATYPE_DERIVED  = 51
 };
 
+/* We start with an arbitrarily chosen number (42), to help with
+ * debugging when a packet type is not initialized or wrongly
+ * initialized. */
 enum MPID_Lock_state {
-    MPID_LOCK_NONE              = 0,
-    MPID_LOCK_SHARED_ALL        = 1
+    MPID_LOCK_NONE              = 42,
+    MPID_LOCK_SHARED_ALL
 };
 
 /*
@@ -190,6 +193,7 @@ static inline int MPIDI_CH3I_RMA_Ops_alloc_tail(MPIDI_RMA_Ops_list_t *list,
 
     tmp_ptr->next = NULL;
     tmp_ptr->dataloop = NULL;
+    tmp_ptr->request = NULL;
 
     MPL_DL_APPEND(*list, tmp_ptr);
 
