@@ -139,7 +139,7 @@ int MPIDI_CH3U_Recvq_init(void)
         MPI_T_BIND_NO_OBJECT,
         (MPIR_T_PVAR_FLAG_READONLY | MPIR_T_PVAR_FLAG_CONTINUOUS),
         "CH3", /* category name */
-        "number of search passes on the message receive queue");
+        "number of search passes on the posted message receive queue");
 
     MPIR_T_PVAR_COUNTER_REGISTER_STATIC(
         RECVQ,
@@ -149,7 +149,7 @@ int MPIDI_CH3U_Recvq_init(void)
         MPI_T_BIND_NO_OBJECT,
         (MPIR_T_PVAR_FLAG_READONLY | MPIR_T_PVAR_FLAG_CONTINUOUS),
         "CH3",
-        "number of search passes on the message receive queue");
+        "number of search passes on the unexpected message receive queue");
 
     MPIR_T_PVAR_TIMER_REGISTER_STATIC(
         RECVQ,
@@ -494,7 +494,8 @@ MPID_Request * MPIDI_CH3U_Recvq_FDU_or_AEP(int source, int tag,
                                            int context_id, MPID_Comm *comm, void *user_buf,
                                            int user_count, MPI_Datatype datatype, int * foundp)
 {
-    int found;
+    int mpi_errno = MPI_SUCCESS;
+    int found = FALSE;
     MPID_Request *rreq, *prev_rreq;
     MPIDI_Message_match match;
     MPIDI_Message_match mask;
@@ -556,8 +557,15 @@ MPID_Request * MPIDI_CH3U_Recvq_FDU_or_AEP(int source, int tag,
 	else {
 	    if (tag == MPI_ANY_TAG)
 		match.parts.tag = mask.parts.tag = 0;
-	    if (source == MPI_ANY_SOURCE)
-		match.parts.rank = mask.parts.rank = 0;
+            if (source == MPI_ANY_SOURCE) {
+                if (!MPIDI_CH3I_Comm_AS_enabled(comm)) {
+                    MPIU_ERR_SET(mpi_errno, MPIX_ERR_PROC_FAILED, "**comm_fail");
+                    rreq->status.MPI_ERROR = mpi_errno;
+                    MPIDI_CH3U_Request_complete(rreq);
+                    goto lock_exit;
+                }
+                match.parts.rank = mask.parts.rank = 0;
+            }
 
 	    do {
             MPIR_T_PVAR_COUNTER_INC(RECVQ, unexpected_recvq_match_attempts, 1);
@@ -594,8 +602,6 @@ MPID_Request * MPIDI_CH3U_Recvq_FDU_or_AEP(int source, int tag,
     /* A matching request was not found in the unexpected queue, so we 
        need to allocate a new request and add it to the posted queue */
     {
-	int mpi_errno = MPI_SUCCESS;
-
         found = FALSE;
 
 	MPIDI_Request_create_rreq( rreq, mpi_errno, goto lock_exit );
@@ -627,13 +633,13 @@ MPID_Request * MPIDI_CH3U_Recvq_FDU_or_AEP(int source, int tag,
             MPIDI_VC_t *vc;
             MPIDI_Comm_get_vc(comm, source, &vc);
             if (vc->state == MPIDI_VC_STATE_MORIBUND) {
-                MPIU_ERR_SET1(mpi_errno, MPIX_ERR_PROC_FAIL_STOP, "**comm_fail", "**comm_fail %d", vc->pg_rank);
+                MPIU_ERR_SET1(mpi_errno, MPIX_ERR_PROC_FAILED, "**comm_fail", "**comm_fail %d", vc->pg_rank);
                 rreq->status.MPI_ERROR = mpi_errno;
                 MPIDI_CH3U_Request_complete(rreq);
                 goto lock_exit;
             }
         } else if (!MPIDI_CH3I_Comm_AS_enabled(comm)) {
-            MPIU_ERR_SET(mpi_errno, MPIX_ERR_PROC_FAIL_STOP, "**comm_fail");
+            MPIU_ERR_SET(mpi_errno, MPIX_ERR_PROC_FAILED, "**comm_fail");
             rreq->status.MPI_ERROR = mpi_errno;
             MPIDI_CH3U_Request_complete(rreq);
             goto lock_exit;
@@ -693,6 +699,7 @@ int MPIDI_CH3U_Recvq_DP(MPID_Request * rreq)
     MPIR_T_PVAR_TIMER_START(RECVQ, time_failed_matching_postedq);
     cur_rreq = recvq_posted_head;
     while (cur_rreq != NULL) {
+        MPIR_T_PVAR_COUNTER_INC(RECVQ, posted_recvq_match_attempts, 1);
 	if (cur_rreq == rreq) {
 	    if (prev_rreq != NULL) {
 		prev_rreq->dev.next = cur_rreq->dev.next;
@@ -802,6 +809,27 @@ MPID_Request * MPIDI_CH3U_Recvq_FDP_or_AEU(MPIDI_Message_match * match,
     }
     MPIR_T_PVAR_TIMER_END(RECVQ, time_failed_matching_postedq);
 
+    /* If we didn't match the request, look to see if the communicator is
+     * revoked. If so, just throw this request away since it won't be used
+     * anyway. */
+    {
+        MPID_Comm *comm_ptr;
+        int mpi_errno ATTRIBUTE((unused)) = MPI_SUCCESS;
+
+        MPIDI_CH3I_Comm_find(match->parts.context_id, &comm_ptr);
+
+        if (comm_ptr && comm_ptr->revoked && MPIR_TAG_MASK_ERROR_BIT(match->parts.tag) != MPIR_AGREE_TAG &&
+                        comm_ptr->revoked && MPIR_TAG_MASK_ERROR_BIT(match->parts.tag) != MPIR_SHRINK_TAG) {
+            *foundp = FALSE;
+            MPIDI_Request_create_null_rreq( rreq, mpi_errno, found=FALSE;goto lock_exit );
+            MPIU_Assert(mpi_errno == MPI_SUCCESS);
+
+            MPIU_DBG_MSG_FMT(CH3_OTHER, VERBOSE,
+                (MPIU_DBG_FDEST, "RECEIVED MESSAGE FOR REVOKED COMM (tag=%d,src=%d,cid=%d)\n", MPIR_TAG_MASK_ERROR_BIT(match->parts.tag), match->parts.rank, comm_ptr->context_id));
+            return rreq;
+        }
+    }
+
     /* A matching request was not found in the posted queue, so we 
        need to allocate a new request and add it to the unexpected queue */
     {
@@ -854,26 +882,20 @@ static inline int req_uses_vc(const MPID_Request* req, const MPIDI_VC_t *vc)
 #define FCNAME MPIU_QUOTE(FUNCNAME)
 /* This dequeues req from the posted recv queue, set req's error code to comm_fail, and updates the req pointer.
    Note that this creates a new error code if one hasn't already been created (i.e., if *error is MPI_SUCCESS). */
-static inline void dequeue_and_set_error(MPID_Request **req,  MPID_Request *prev_req, int *error, int rank)
+static inline void dequeue_and_set_error(MPID_Request **req,  MPID_Request *prev_req, MPID_Request **head, MPID_Request **tail, int *error, int rank)
 {
     MPID_Request *next = (*req)->dev.next;
-
-    if (*error == MPI_SUCCESS) {
-        if (rank == MPI_PROC_NULL)
-            MPIU_ERR_SET(*error, MPIX_ERR_PROC_FAIL_STOP, "**comm_fail");
-        else
-            MPIU_ERR_SET1(*error, MPIX_ERR_PROC_FAIL_STOP, "**comm_fail", "**comm_fail %d", rank);
-    }
     
     /* remove from queue */
-    if (recvq_posted_head == *req)
-        recvq_posted_head = (*req)->dev.next;
-    else
-        prev_req->dev.next = (*req)->dev.next;
-    if (recvq_posted_tail == *req)
-        recvq_posted_tail = prev_req;
+    if (*head == *req) {
+        if (*head == recvq_posted_head) MPIR_T_PVAR_LEVEL_DEC(RECVQ, posted_recvq_length, 1);
 
-    MPIR_T_PVAR_LEVEL_DEC(RECVQ, posted_recvq_length, 1);
+        *head = (*req)->dev.next;
+    } else
+        prev_req->dev.next = (*req)->dev.next;
+
+    if (*tail == *req)
+        *tail = prev_req;
 
     /* set error and complete */
     (*req)->status.MPI_ERROR = *error;
@@ -884,6 +906,138 @@ static inline void dequeue_and_set_error(MPID_Request **req,  MPID_Request *prev
     *req = next;
 }
 
+/*
+ * MPIDI_CH3U_Clean_recvq()
+ *
+ * Looks through the entire unexpected recv queue and the posted recv queues.
+ * If a request is found that involved the provided communicator (comm_ptr),
+ * it is dequeed and marked as failed via MPIX_ERR_REVOKED.
+ *
+ * Multithread - This routine must be called from within a MSGQUEUE
+ * critical section.  If a request is allocated, it must not release
+ * the MSGQUEUE until the request is completely valid, as another thread
+ * may then find it and dequeue it.
+ *
+ */
+int MPIDI_CH3U_Clean_recvq(MPID_Comm *comm_ptr)
+{
+    int mpi_errno = MPI_SUCCESS;
+    int error = MPI_SUCCESS;
+    MPID_Request *rreq, *prev_rreq = NULL;
+    MPIDI_Message_match match;
+    MPIDI_Message_match mask;
+    MPIDI_STATE_DECL(MPIDI_CH3U_CLEAN_RECVQ);
+
+    MPIDI_FUNC_ENTER(MPIDI_CH3U_CLEAN_RECVQ);
+
+    MPIU_THREAD_CS_ASSERT_HELD(MSGQUEUE);
+
+    MPIU_ERR_SETSIMPLE(error, MPIX_ERR_REVOKED, "**revoked");
+
+    rreq = recvq_unexpected_head;
+    mask.parts.context_id = ~0;
+    mask.parts.rank = mask.parts.tag = 0;
+
+    /* Clear the error bit in the tag since we don't care about whether or
+     * not we're trying to report an error anymore. */
+    MPIR_TAG_CLEAR_ERROR_BIT(mask.parts.tag);
+
+    while (NULL != rreq) {
+        /* We'll have to do this matching twice. Once for the pt2pt context id
+         * and once for the collective context id */
+        match.parts.context_id = comm_ptr->recvcontext_id + MPID_CONTEXT_INTRA_PT2PT;
+
+        if (MATCH_WITH_LEFT_RIGHT_MASK(rreq->dev.match, match, mask)) {
+            MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
+                        "cleaning up unexpected pt2pt pkt rank=%d tag=%d contextid=%d",
+                        rreq->dev.match.parts.rank, rreq->dev.match.parts.tag, rreq->dev.match.parts.context_id));
+            dequeue_and_set_error(&rreq, prev_rreq, &recvq_unexpected_head, &recvq_unexpected_tail, &error, MPI_PROC_NULL);
+            continue;
+        }
+
+        match.parts.context_id = comm_ptr->recvcontext_id + MPID_CONTEXT_INTRA_COLL;
+
+        if (MATCH_WITH_LEFT_RIGHT_MASK(rreq->dev.match, match, mask)) {
+            if (rreq->dev.match.parts.tag != MPIR_AGREE_TAG && rreq->dev.match.parts.tag != MPIR_SHRINK_TAG) {
+                MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
+                            "cleaning up unexpected collective pkt rank=%d tag=%d contextid=%d",
+                            rreq->dev.match.parts.rank, rreq->dev.match.parts.tag, rreq->dev.match.parts.context_id));
+                dequeue_and_set_error(&rreq, prev_rreq, &recvq_unexpected_head, &recvq_unexpected_tail, &error, MPI_PROC_NULL);
+                continue;
+            }
+        }
+
+        if (MPIR_CVAR_ENABLE_SMP_COLLECTIVES && MPIR_Comm_is_node_aware(comm_ptr)) {
+            int offset = (comm_ptr->comm_kind == MPID_INTRACOMM) ?  MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+            match.parts.context_id = comm_ptr->recvcontext_id + MPID_CONTEXT_INTRANODE_OFFSET + offset;
+
+            if (MATCH_WITH_LEFT_RIGHT_MASK(rreq->dev.match, match, mask)) {
+                if (rreq->dev.match.parts.tag != MPIR_AGREE_TAG && rreq->dev.match.parts.tag != MPIR_SHRINK_TAG) {
+                    MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
+                                "cleaning up unexpected collective pkt rank=%d tag=%d contextid=%d",
+                                rreq->dev.match.parts.rank, rreq->dev.match.parts.tag, rreq->dev.match.parts.context_id));
+                    dequeue_and_set_error(&rreq, prev_rreq, &recvq_unexpected_head, &recvq_unexpected_tail, &error, MPI_PROC_NULL);
+                    continue;
+                }
+            }
+        }
+
+        prev_rreq = rreq;
+        rreq = rreq->dev.next;
+    }
+
+    rreq = recvq_posted_head;
+    prev_rreq = NULL;
+
+    while (NULL != rreq) {
+        /* We'll have to do this matching twice. Once for the pt2pt context id
+         * and once for the collective context id */
+        match.parts.context_id = comm_ptr->recvcontext_id + MPID_CONTEXT_INTRA_PT2PT;
+
+        if (MATCH_WITH_LEFT_RIGHT_MASK(rreq->dev.match, match, mask)) {
+            MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
+                        "cleaning up posted pt2pt pkt rank=%d tag=%d contextid=%d",
+                        rreq->dev.match.parts.rank, rreq->dev.match.parts.tag, rreq->dev.match.parts.context_id));
+            dequeue_and_set_error(&rreq, prev_rreq, &recvq_posted_head, &recvq_posted_tail, &error, MPI_PROC_NULL);
+            continue;
+        }
+
+        match.parts.context_id = comm_ptr->recvcontext_id + MPID_CONTEXT_INTRA_COLL;
+
+        if (MATCH_WITH_LEFT_RIGHT_MASK(rreq->dev.match, match, mask)) {
+            if (rreq->dev.match.parts.tag != MPIR_AGREE_TAG && rreq->dev.match.parts.tag != MPIR_SHRINK_TAG) {
+                MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
+                            "cleaning up posted collective pkt rank=%d tag=%d contextid=%d",
+                            rreq->dev.match.parts.rank, rreq->dev.match.parts.tag, rreq->dev.match.parts.context_id));
+                dequeue_and_set_error(&rreq, prev_rreq, &recvq_posted_head, &recvq_posted_tail, &error, MPI_PROC_NULL);
+                continue;
+            }
+        }
+
+        if (MPIR_CVAR_ENABLE_SMP_COLLECTIVES && MPIR_Comm_is_node_aware(comm_ptr)) {
+            int offset = (comm_ptr->comm_kind == MPID_INTRACOMM) ?  MPID_CONTEXT_INTRA_COLL : MPID_CONTEXT_INTER_COLL;
+            match.parts.context_id = comm_ptr->recvcontext_id + MPID_CONTEXT_INTRANODE_OFFSET + offset;
+
+            if (MATCH_WITH_LEFT_RIGHT_MASK(rreq->dev.match, match, mask)) {
+                if (rreq->dev.match.parts.tag != MPIR_AGREE_TAG && rreq->dev.match.parts.tag != MPIR_SHRINK_TAG) {
+                    MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
+                                "cleaning up posted collective pkt rank=%d tag=%d contextid=%d",
+                                rreq->dev.match.parts.rank, rreq->dev.match.parts.tag, rreq->dev.match.parts.context_id));
+                    dequeue_and_set_error(&rreq, prev_rreq, &recvq_posted_head, &recvq_posted_tail, &error, MPI_PROC_NULL);
+                    continue;
+                }
+            }
+        }
+
+        prev_rreq = rreq;
+        rreq = rreq->dev.next;
+    }
+
+    MPIDI_FUNC_EXIT(MPIDI_CH3U_CLEAN_RECVQ);
+
+    return mpi_errno;
+}
+
 #undef FUNCNAME
 #define FUNCNAME MPIDI_CH3U_Complete_disabled_anysources
 #undef FCNAME
@@ -897,7 +1051,9 @@ int MPIDI_CH3U_Complete_disabled_anysources(void)
 
     MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_CH3U_COMPLETE_DISABLED_ANYSOURCES);
     MPIU_THREAD_CS_ENTER(MSGQUEUE,);
-    
+
+    MPIU_ERR_SETSIMPLE(error, MPIX_ERR_PROC_FAILED_PENDING, "**failure_pending");
+
     /* Check each request in the posted queue, and complete-with-error any
        anysource requests posted on communicators that have disabled
        anysources */
@@ -905,7 +1061,7 @@ int MPIDI_CH3U_Complete_disabled_anysources(void)
     prev_req = NULL;
     while (req) {
         if (req->dev.match.parts.rank == MPI_ANY_SOURCE && !MPIDI_CH3I_Comm_AS_enabled(req->comm)) {
-            dequeue_and_set_error(&req, prev_req, &error, MPI_PROC_NULL); /* we don't know the rank of the failed proc */
+            dequeue_and_set_error(&req, prev_req, &recvq_posted_head, &recvq_posted_tail, &error, MPI_PROC_NULL); /* we don't know the rank of the failed proc */
         } else {
             prev_req = req;
             req = req->dev.next;
@@ -937,13 +1093,15 @@ int MPIDI_CH3U_Complete_posted_with_error(MPIDI_VC_t *vc)
 
     MPIU_THREAD_CS_ENTER(MSGQUEUE,);
 
+    MPIU_ERR_SETSIMPLE(error, MPIX_ERR_PROC_FAILED, "**proc_failed");
+
     /* check each req in the posted queue and complete-with-error any requests
        using this VC. */
     req = recvq_posted_head;
     prev_req = NULL;
     while (req) {
         if (req->dev.match.parts.rank != MPI_ANY_SOURCE && req_uses_vc(req, vc)) {
-            dequeue_and_set_error(&req, prev_req, &error, vc->pg_rank);
+            dequeue_and_set_error(&req, prev_req, &recvq_posted_head, &recvq_posted_tail, &error, MPI_PROC_NULL);
         } else {
             prev_req = req;
             req = req->dev.next;
