@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
  *  (C) 2001 by Argonne National Laboratory.
  *      See COPYRIGHT in top-level directory.
@@ -27,6 +27,7 @@ int MPID_Issend(const void * buf, int count, MPI_Datatype datatype, int rank, in
 #if defined(MPID_USE_SEQUENCE_NUMBERS)
     MPID_Seqnum_t seqnum;
 #endif    
+    int eager_threshold = -1;
     int mpi_errno = MPI_SUCCESS;
     MPIDI_STATE_DECL(MPID_STATE_MPID_ISSEND);
 
@@ -35,6 +36,13 @@ int MPID_Issend(const void * buf, int count, MPI_Datatype datatype, int rank, in
     MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
                  "rank=%d, tag=%d, context=%d", 
                  rank, tag, comm->context_id + context_offset));
+
+    /* Check to make sure the communicator hasn't already been revoked */
+    if (comm->revoked &&
+            MPIR_AGREE_TAG != MPIR_TAG_MASK_ERROR_BIT(tag & ~MPIR_Process.tagged_coll_mask) &&
+            MPIR_SHRINK_TAG != MPIR_TAG_MASK_ERROR_BIT(tag & ~MPIR_Process.tagged_coll_mask)) {
+        MPIU_ERR_SETANDJUMP(mpi_errno,MPIX_ERR_REVOKED,"**revoked");
+    }
     
     if (rank == comm->rank && comm->comm_kind != MPID_INTERCOMM)
     {
@@ -62,7 +70,7 @@ int MPID_Issend(const void * buf, int count, MPI_Datatype datatype, int rank, in
     if (rank == MPI_PROC_NULL)
     {
 	MPIU_Object_set_ref(sreq, 1);
-	sreq->cc = 0;
+        MPID_cc_set(&sreq->cc, 0);
 	goto fn_exit;
     }
     
@@ -74,8 +82,10 @@ int MPID_Issend(const void * buf, int count, MPI_Datatype datatype, int rank, in
 					     context_offset );
 	goto fn_exit;
     }
-    
-    if (data_sz + sizeof(MPIDI_CH3_Pkt_eager_sync_send_t) <= vc->eager_max_msg_sz)
+
+    MPIDI_CH3_GET_EAGER_THRESHOLD(&eager_threshold, comm, vc);
+
+    if (data_sz + sizeof(MPIDI_CH3_Pkt_eager_sync_send_t) <= eager_threshold)
     {
 	mpi_errno = MPIDI_CH3_EagerSyncNoncontigSend( &sreq, buf, count,
                                                       datatype, data_sz, 
@@ -121,6 +131,7 @@ int MPID_Issend(const void * buf, int count, MPI_Datatype datatype, int rank, in
     }
 		  )
     
+  fn_fail:
     MPIDI_FUNC_EXIT(MPID_STATE_MPID_ISSEND);
     return mpi_errno;
 }
