@@ -39,6 +39,13 @@ int MPID_Rsend(const void * buf, int count, MPI_Datatype datatype, int rank, int
     MPIU_DBG_MSG_FMT(CH3_OTHER,VERBOSE,(MPIU_DBG_FDEST,
 					"rank=%d, tag=%d, context=%d", 
                               rank, tag, comm->context_id + context_offset));
+
+    /* Check to make sure the communicator hasn't already been revoked */
+    if (comm->revoked &&
+            MPIR_AGREE_TAG != MPIR_TAG_MASK_ERROR_BIT(tag & ~MPIR_Process.tagged_coll_mask) &&
+            MPIR_SHRINK_TAG != MPIR_TAG_MASK_ERROR_BIT(tag & ~MPIR_Process.tagged_coll_mask)) {
+        MPIU_ERR_SETANDJUMP(mpi_errno,MPIX_ERR_REVOKED,"**revoked");
+    }
     
     if (rank == comm->rank && comm->comm_kind != MPID_INTERCOMM)
     {
@@ -151,6 +158,7 @@ int MPID_Rsend(const void * buf, int count, MPI_Datatype datatype, int rank, int
     }
 		  );
     
+  fn_fail:
     MPIDI_FUNC_EXIT(MPID_STATE_MPID_RSEND);
     return mpi_errno;
 }
