@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2012 inria.  All rights reserved.
+ * Copyright © 2009-2014 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux 1
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -85,66 +85,6 @@ hwloc_calc_get_obj_inside_cpuset_by_depth(hwloc_topology_t topology, hwloc_const
   }
 }
 
-/* extended version of hwloc_obj_type_of_string()
- *
- * matches L2, L3Cache and Group4, and return the corresponding depth attribute if depthattrp isn't NULL.
- * only looks at the beginning of the string to allow truncated type names.
- */
-static __hwloc_inline int
-hwloc_obj_type_sscanf(const char *string, hwloc_obj_type_t *typep, int *depthattrp, hwloc_obj_cache_type_t *cachetypeattrp)
-{
-  hwloc_obj_type_t type = (hwloc_obj_type_t) -1;
-  int depthattr = -1;
-  hwloc_obj_cache_type_t cachetypeattr = (hwloc_obj_cache_type_t) -1; /* unspecified */
-
-  /* types without depthattr */
-  if (!hwloc_strncasecmp(string, "system", 2)) {
-    type = HWLOC_OBJ_SYSTEM;
-  } else if (!hwloc_strncasecmp(string, "machine", 2)) {
-    type = HWLOC_OBJ_MACHINE;
-  } else if (!hwloc_strncasecmp(string, "node", 1)) {
-    type = HWLOC_OBJ_NODE;
-  } else if (!hwloc_strncasecmp(string, "socket", 2)) {
-    type = HWLOC_OBJ_SOCKET;
-  } else if (!hwloc_strncasecmp(string, "core", 2)) {
-    type = HWLOC_OBJ_CORE;
-  } else if (!hwloc_strncasecmp(string, "pu", 2)) {
-    type = HWLOC_OBJ_PU;
-  } else if (!hwloc_strncasecmp(string, "misc", 2)) {
-    type = HWLOC_OBJ_MISC;
-  } else if (!hwloc_strncasecmp(string, "pci", 2)) {
-    type = HWLOC_OBJ_PCI_DEVICE;
-  } else if (!hwloc_strncasecmp(string, "os", 2)) {
-    type = HWLOC_OBJ_OS_DEVICE;
-
-  /* types with depthattr */
-  } else if (!hwloc_strncasecmp(string, "cache", 2)) {
-    type = HWLOC_OBJ_CACHE;
-  } else if ((string[0] == 'l' || string[0] == 'L') && string[1] >= '0' && string[1] <= '9') {
-    type = HWLOC_OBJ_CACHE;
-    depthattr = atoi(string+1);
-    if (string[2] == 'd')
-      cachetypeattr = HWLOC_OBJ_CACHE_DATA;
-    else if (string[2] == 'i')
-      cachetypeattr = HWLOC_OBJ_CACHE_INSTRUCTION;
-  } else if (!hwloc_strncasecmp(string, "group", 1)) {
-    int length;
-    type = HWLOC_OBJ_GROUP;
-    length = strcspn(string, "0123456789");
-    if (string[length] != '\0')
-      depthattr = atoi(string+length);
-  } else
-    return -1;
-
-  *typep = type;
-  if (depthattrp)
-    *depthattrp = depthattr;
-  if (cachetypeattrp)
-    *cachetypeattrp = cachetypeattr;
-
-  return 0;
-}
-
 static __hwloc_inline int
 hwloc_calc_depth_of_type(hwloc_topology_t topology, hwloc_obj_type_t type,
 			 int depthattr, hwloc_obj_cache_type_t cachetype /* -1 if not specified */,
@@ -229,7 +169,7 @@ hwloc_calc_parse_depth_prefix(hwloc_topology_t topology, unsigned topodepth,
   typestring[typelen] = '\0';
 
   /* try to match a type name */
-  err = hwloc_obj_type_sscanf(typestring, &type, &depthattr, &cachetypeattr);
+  err = hwloc_obj_type_sscanf(typestring, &type, &depthattr, &cachetypeattr, sizeof(cachetypeattr));
   if (!err) {
     *typep = type;
     return hwloc_calc_depth_of_type(topology, type, depthattr, cachetypeattr, verbose);
@@ -252,15 +192,29 @@ hwloc_calc_parse_depth_prefix(hwloc_topology_t topology, unsigned topodepth,
 }
 
 static __hwloc_inline int
-hwloc_calc_parse_range(const char *string,
+hwloc_calc_parse_range(const char *_string,
 		       int *firstp, int *amountp, int *stepp, int *wrapp,
 		       const char **dotp)
 {
-  const char *dash, *dot, *colon;
-  int first, amount, step, wrap;
+  char string[65];
+  size_t len;
+  char *dot, *end, *end2;
+  long first, last, amount;
+  int wrap;
 
-  dot = strchr(string, '.');
+  dot = strchr(_string, '.');
   *dotp = dot;
+  if (dot) {
+    len = dot - _string;
+  } else {
+    len = strlen(_string);
+  }
+  if (len >= sizeof(string)) {
+    fprintf(stderr, "invalid range `%s', too long\n", string);
+    return -1;
+  }
+  memcpy(string, _string, len);
+  string[len] = '\0';
 
   if (!isdigit(*string)) {
     if (!strncmp(string, "all", 3)) {
@@ -281,32 +235,49 @@ hwloc_calc_parse_range(const char *string,
       *stepp = 2;
       *wrapp = 0;
       return 0;
-    } else
+    } else {
+      fprintf(stderr, "unrecognized range keyword `%s'\n", string);
       return -1;
+    }
   }
 
-  first = atoi(string);
+  first = strtol(string, &end, 10);
   amount = 1;
-  step = 1;
   wrap = 0;
 
-  dash = strchr(string, '-');
-  if (dash && (dash < dot || !dot)) {
-    if (*(dash+1) == '\0')
+  if (*end == '-') {
+    last = strtol(end+1, &end2, 10);
+    if (*end2) {
+      fprintf(stderr, "invalid character at `%s' after range at `%s'\n", end2, string);
+      return -1;
+    } else if (end2 == end+1) {
+      /* X- */
       amount = -1;
-    else
-      amount = atoi(dash+1)-first+1;
-  } else {
-    colon = strchr(string, ':');
-    if (colon && (colon < dot || !dot)) {
-      amount = atoi(colon+1);
-      wrap = 1;
+    } else {
+      /* X-Y */
+      amount = last-first+1;
     }
+
+  } else if (*end == ':') {
+    /* X:Y */
+    wrap = 1;
+    amount = strtol(end+1, &end2, 10);
+    if (*end2) {
+      fprintf(stderr, "invalid character at `%s' after range at `%s'\n", end2, string);
+      return -1;
+    } else if (end2 == end+1) {
+      fprintf(stderr, "missing width at `%s' in range at `%s'\n", end2, string);
+      return -1;
+    }
+
+  } else if (*end) {
+    fprintf(stderr, "invalid character at `%s' after index at `%s'\n", end, string);
+    return -1;
   }
 
   *firstp = first;
   *amountp = amount;
-  *stepp = step;
+  *stepp = 1;
   *wrapp = wrap;
   return 0;
 }
@@ -315,7 +286,9 @@ static __hwloc_inline int
 hwloc_calc_append_object_range(hwloc_topology_t topology, unsigned topodepth,
 			       hwloc_const_bitmap_t rootset, int depth,
 			       const char *string, /* starts with indexes following the colon */
-			       int logical, hwloc_bitmap_t set, int verbose)
+			       int logical,
+			       void (*cbfunc)(void *, hwloc_obj_t, int), void *cbdata,
+			       int verbose)
 {
   hwloc_obj_t obj;
   unsigned width;
@@ -372,12 +345,12 @@ hwloc_calc_append_object_range(hwloc_topology_t topology, unsigned topodepth,
     }
     if (obj) {
       if (dot) {
-	hwloc_calc_append_object_range(topology, topodepth, obj->cpuset, nextdepth, nextsep+1, logical, set, verbose);
+	hwloc_calc_append_object_range(topology, topodepth, obj->cpuset, nextdepth, nextsep+1, logical, cbfunc, cbdata, verbose);
       } else {
 	/* add to the temporary cpuset
 	 * and let the caller add/clear/and/xor for the actual final cpuset depending on cmdline options
 	 */
-        hwloc_calc_append_cpuset(set, obj->cpuset, HWLOC_CALC_APPEND_ADD, verbose);
+        cbfunc(cbdata, obj, verbose);
       }
     }
   }
@@ -386,15 +359,11 @@ hwloc_calc_append_object_range(hwloc_topology_t topology, unsigned topodepth,
 }
 
 static __hwloc_inline int
-hwloc_calc_append_iodev(hwloc_bitmap_t set, hwloc_obj_t obj,
-			hwloc_calc_append_mode_t mode, int verbose)
+hwloc_calc_append_iodev(void (*cbfunc)(void *, hwloc_obj_t, int), void *cbdata,
+			hwloc_obj_t obj,
+			int verbose)
 {
-  while (obj && !obj->cpuset)
-    obj = obj->parent;
-  if (!obj)
-    /* do nothing */
-    return 0;
-  hwloc_calc_append_cpuset(set, obj->cpuset, mode, verbose);
+  cbfunc(cbdata, obj, verbose);
   return 0;
 }
 
@@ -425,7 +394,9 @@ hwloc_calc_find_next_pci_object(hwloc_topology_t topology, int vendor, int devic
 }
 
 static __hwloc_inline int
-hwloc_calc_append_pci_object_range(hwloc_topology_t topology, const char *string, hwloc_bitmap_t set, int verbose)
+hwloc_calc_append_pci_object_range(hwloc_topology_t topology, const char *string,
+				   void (*cbfunc)(void *, hwloc_obj_t, int), void *cbdata,
+				   int verbose)
 {
   hwloc_obj_t obj;
   int vendor, device;
@@ -472,7 +443,7 @@ hwloc_calc_append_pci_object_range(hwloc_topology_t topology, const char *string
       if (verbose > 0)
 	printf("using matching PCI object #%d bus id %04x:%02x:%02x.%01x\n", i,
 	       obj->attr->pcidev.domain, obj->attr->pcidev.bus, obj->attr->pcidev.dev, obj->attr->pcidev.func);
-      hwloc_calc_append_iodev(set, obj, HWLOC_CALC_APPEND_ADD, verbose);
+      hwloc_calc_append_iodev(cbfunc, cbdata, obj, verbose);
     } else {
       if (amount != -1)
 	if (verbose >= 0)
@@ -492,6 +463,67 @@ hwloc_calc_append_pci_object_range(hwloc_topology_t topology, const char *string
   return -1;
 }
 
+static __hwloc_inline int
+hwloc_calc_process_type_arg(hwloc_topology_t topology, unsigned topodepth,
+			    const char *arg, size_t typelen,
+			    int logical,
+			    void (*cbfunc)(void *, hwloc_obj_t, int), void *cbdata,
+			    int verbose)
+{
+  const char *sep = &arg[typelen];
+  hwloc_obj_type_t type = (hwloc_obj_type_t) -1;
+  int depth;
+
+  depth = hwloc_calc_parse_depth_prefix(topology, topodepth,
+					arg, typelen,
+					&type,
+					verbose);
+  if (depth < 0) {
+    /* if we didn't find a depth but found a type, handle special cases */
+    hwloc_obj_t obj = NULL;
+    if (*sep == ':' && type == HWLOC_OBJ_PCI_DEVICE) {
+      /* FIXME: change to another syntax? */
+      return hwloc_calc_append_pci_object_range(topology, sep+1, cbfunc, cbdata, verbose);
+
+    } else if (*sep == '=' && type == HWLOC_OBJ_PCI_DEVICE) {
+      /* try to match a busid */
+      obj = hwloc_get_pcidev_by_busidstring(topology, sep+1);
+      if (obj)
+	return hwloc_calc_append_iodev(cbfunc, cbdata, obj, verbose);
+      if (verbose >= 0)
+	fprintf(stderr, "invalid PCI device %s\n", sep+1);
+      return -1;
+
+    } else if (*sep == '=' && type == HWLOC_OBJ_OS_DEVICE) {
+      /* try to match a OS device name */
+      while ((obj = hwloc_get_next_osdev(topology, obj)) != NULL) {
+	if (!strcmp(obj->name, sep+1))
+	  return hwloc_calc_append_iodev(cbfunc, cbdata, obj, verbose);
+      }
+      if (verbose >= 0)
+	fprintf(stderr, "invalid OS device %s\n", sep+1);
+      return -1;
+    } else
+      return -1;
+  }
+
+  /* look at indexes following this type/depth */
+  return hwloc_calc_append_object_range(topology, topodepth, hwloc_topology_get_complete_cpuset(topology), depth, sep+1, logical, cbfunc, cbdata, verbose);
+}
+
+static __hwloc_inline void
+hwloc_calc_process_arg_cpuset_cb(void *_data, hwloc_obj_t obj, int verbose)
+{
+  hwloc_bitmap_t set = _data;
+  /* walk up out of I/O objects */
+  while (obj && !obj->cpuset)
+    obj = obj->parent;
+  if (!obj)
+    /* do nothing */
+    return;
+  hwloc_calc_append_cpuset(set, obj->cpuset, HWLOC_CALC_APPEND_ADD, verbose);
+}
+
 static __hwloc_inline int
 hwloc_calc_process_arg(hwloc_topology_t topology, unsigned topodepth,
 		       const char *arg, int logical, hwloc_bitmap_t set,
@@ -518,52 +550,11 @@ hwloc_calc_process_arg(hwloc_topology_t topology, unsigned topodepth,
   /* try to match a type/depth followed by a special character */
   typelen = strspn(arg, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
   if (typelen && (arg[typelen] == ':' || arg[typelen] == '=')) {
-    const char *sep = &arg[typelen];
-    hwloc_obj_type_t type = (hwloc_obj_type_t) -1;
-    int depth;
-    hwloc_bitmap_t newset;
-
-    depth = hwloc_calc_parse_depth_prefix(topology, topodepth,
-					  arg, typelen,
-					  &type,
-					  verbose);
-    if (depth < 0) {
-      /* if we didn't find a depth but found a type, handle special cases */
-      hwloc_obj_t obj = NULL;
-      if (*sep == ':' && type == HWLOC_OBJ_PCI_DEVICE) {
-	/* FIXME: change to another syntax? */
-	/* agregate the whole range in newset before applying it to set */
-	newset = hwloc_bitmap_alloc();
-	err = hwloc_calc_append_pci_object_range(topology, sep+1, newset, verbose);
-	if (!err)
-	  err = hwloc_calc_append_cpuset(set, newset, mode, verbose);
-	hwloc_bitmap_free(newset);
-	return err;
-      } else if (*sep == '=' && type == HWLOC_OBJ_PCI_DEVICE) {
-	/* try to match a busid */
-	obj = hwloc_get_pcidev_by_busidstring(topology, sep+1);
-	if (obj)
-	  return hwloc_calc_append_iodev(set, obj, mode, verbose);
-	if (verbose >= 0)
-	  fprintf(stderr, "invalid PCI device %s\n", sep+1);
-	return -1;
-      } else if (*sep == '=' && type == HWLOC_OBJ_OS_DEVICE) {
-	/* try to match a OS device name */
-	while ((obj = hwloc_get_next_osdev(topology, obj)) != NULL) {
-	  if (!strcmp(obj->name, sep+1))
-	    return hwloc_calc_append_iodev(set, obj, mode, verbose);
-	}
-	if (verbose >= 0)
-	  fprintf(stderr, "invalid OS device %s\n", sep+1);
-	return -1;
-      } else
-	return -1;
-    }
-
-    /* look at indexes following this type/depth */
-    /* agregate the whole range in newset before applying it to set */
-    newset = hwloc_bitmap_alloc();
-    err = hwloc_calc_append_object_range(topology, topodepth, hwloc_topology_get_complete_cpuset(topology), depth, sep+1, logical, newset, verbose);
+    /* process type/depth */
+    hwloc_bitmap_t newset = hwloc_bitmap_alloc();
+    err = hwloc_calc_process_type_arg(topology, topodepth, arg, typelen, logical,
+				      hwloc_calc_process_arg_cpuset_cb, newset,
+				      verbose);
     if (!err)
       err = hwloc_calc_append_cpuset(set, newset, mode, verbose);
     hwloc_bitmap_free(newset);
